#!/bin/bash
#

# build_numpy.sh 

# The build is qualified because it depends upon LAPACK, which is qualified.
usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e1[0457]|c2>:<p2711|p2713d|p2714|p2714b> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && pwd -P`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
    qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
    for qual in "${qlist[@]}"; do
	    case $qual in
        e10|e1[457]|c2)
        	cqual=$qual
        	;;
        p2711|p2713d|p2714|p2714b)
        	pqual=$qual
        	;;
        *)
        	ssi_die "Unrecognized qualifier $qual"
	    esac
    done
    if [[ -z "$cqual" ]]; then
	    ssi_die "Base qualifier is mandatory"
    fi
    if [[ -z "$pqual" ]]; then
	    ssi_die "A python qualifier is mandatory (e.g. p2714b)."
    fi
    basequal=${cqual}:${pqual}
    extraqual=prof
}

set_compiler_flags()
{
  cflags="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
maketar=${3}

if [[ "${maketar:-tar}" != "tar" ]]; then
  echo "WARNING: build type specifier ${maketar} no longer accepted: ignoring." 1>&2
  maketar=${4}
fi

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=numpy
origpkgver=v1_14_3
pkgver=${origpkgver}
ssibuildshims_version=v1_04_04
setuptools_version=v22_0_5 # Required only for python < p2713d.
nose_version=v1_3_7e
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir
get_ssibuildshims

source ssi_functions --
set_qualifiers

source define_basics NO_DEBUG_SOURCE

set_compiler_flags

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "fake setup failed"

if [ ${OS1} = "Darwin" ]
then
   PATH=`dropit /usr/bin/get-cert $PATH`
fi

(( pv = ${pqual//[A-Za-z]/} ))
if (( ${pv} < 2713 )); then # Bundled with newer python.
  setup -B setuptools ${setuptools_version} -q ${pqual} -z ${product_dir}:${PRODUCTS} || \
    ssi_die "unable to set up UPS package for python setuptools ${setuptools_version}"
fi

set -x

# work in a build directory, which will not be part of any distribution
python_shortdotversion=${PYTHON_VERSION#v}
python_shortdotversion=${python_shortdotversion//_/.}
python_shortdotversion=${python_shortdotversion%.*}

mkdir -p ${pkgdir}/lib/python${python_shortdotversion}/site-packages || ssi_die "failed to create ${pkgdir}/lib/python${python_shortdotversion}/site-packages"
mkdir -p ${blddir} || ssi_die "failed to create ${blddir}"

cd ${blddir} || ssi_die "Unable to cd into ${blddir}"
tar xf ${tardir}/${pkgtarname}.tar.gz || ssi_die "Failed to untar ${pkgtarname}.tar.gz"
cd ${package}-${pkgdotver} || ssi_die "Failed to cd into ${package}-${pkgdotver}"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

FC=false \
CC=gcc \
CXX=false \
CFLAGS="${cflags}" \
python setup.py build -j ${ncore} install --prefix ${pkgdir} || ssi_die "python setup.py install failed"

# Run some tests.
setup -B nose $nose_version -q +${pqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || \
  ssi_die "unable to set up nose ${nose_version} -q +${pqual} for tests."
nosetests -w ${pkgdir}/lib/python${python_shortdotversion}/site-packages/numpy*/numpy/tests || ssi_die "nosetests failed"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

#replace hardwired python path 
sed -i'' -e '1s/^.*$/#!\/usr\/bin\/env python /' ${pkgdir}/bin/f2py || ssi_die "Failed to fix shebang for f2py"

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "$package is installed at $(eval echo "\${${package^^*}_FQ_DIR}")"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
